/* This file is automatically generated from cssvalues.in by
#makevalues, do not edit */
/* Copyright 1998 W. Bastian */

#ifndef CSSVALUES_H
#define CSSVALUES_H

DOM::DOMString getValueName(unsigned short id);

#define CSS_VAL_INVALID 0
#define CSS_VAL_MIN 1
#define CSS_VAL_INHERIT 1
#define CSS_VAL_INITIAL 2
#define CSS_VAL_NONE 3
#define CSS_VAL_HIDDEN 4
#define CSS_VAL_INSET 5
#define CSS_VAL_GROOVE 6
#define CSS_VAL_RIDGE 7
#define CSS_VAL_OUTSET 8
#define CSS_VAL_DOTTED 9
#define CSS_VAL_DASHED 10
#define CSS_VAL_SOLID 11
#define CSS_VAL_DOUBLE 12
#define CSS_VAL_CAPTION 13
#define CSS_VAL_ICON 14
#define CSS_VAL_MENU 15
#define CSS_VAL_MESSAGE_BOX 16
#define CSS_VAL_SMALL_CAPTION 17
#define CSS_VAL_STATUS_BAR 18
#define CSS_VAL_ITALIC 19
#define CSS_VAL_OBLIQUE 20
#define CSS_VAL_SMALL_CAPS 21
#define CSS_VAL_NORMAL 22
#define CSS_VAL_BOLD 23
#define CSS_VAL_BOLDER 24
#define CSS_VAL_LIGHTER 25
#define CSS_VAL_100 26
#define CSS_VAL_200 27
#define CSS_VAL_300 28
#define CSS_VAL_400 29
#define CSS_VAL_500 30
#define CSS_VAL_600 31
#define CSS_VAL_700 32
#define CSS_VAL_800 33
#define CSS_VAL_900 34
#define CSS_VAL_XX_SMALL 35
#define CSS_VAL_X_SMALL 36
#define CSS_VAL_SMALL 37
#define CSS_VAL_MEDIUM 38
#define CSS_VAL_LARGE 39
#define CSS_VAL_X_LARGE 40
#define CSS_VAL_XX_LARGE 41
#define CSS_VAL__KHTML_XXX_LARGE 42
#define CSS_VAL_SMALLER 43
#define CSS_VAL_LARGER 44
#define CSS_VAL_WIDER 45
#define CSS_VAL_NARROWER 46
#define CSS_VAL_ULTRA_CONDENSED 47
#define CSS_VAL_EXTRA_CONDENSED 48
#define CSS_VAL_CONDENSED 49
#define CSS_VAL_SEMI_CONDENSED 50
#define CSS_VAL_SEMI_EXPANDED 51
#define CSS_VAL_EXPANDED 52
#define CSS_VAL_EXTRA_EXPANDED 53
#define CSS_VAL_ULTRA_EXPANDED 54
#define CSS_VAL_SERIF 55
#define CSS_VAL_SANS_SERIF 56
#define CSS_VAL_CURSIVE 57
#define CSS_VAL_FANTASY 58
#define CSS_VAL_MONOSPACE 59
#define CSS_VAL__KHTML_BODY 60
#define CSS_VAL_AQUA 61
#define CSS_VAL_BLACK 62
#define CSS_VAL_BLUE 63
#define CSS_VAL_FUCHSIA 64
#define CSS_VAL_GRAY 65
#define CSS_VAL_GREEN 66
#define CSS_VAL_LIME 67
#define CSS_VAL_MAROON 68
#define CSS_VAL_NAVY 69
#define CSS_VAL_OLIVE 70
#define CSS_VAL_ORANGE 71
#define CSS_VAL_PURPLE 72
#define CSS_VAL_RED 73
#define CSS_VAL_SILVER 74
#define CSS_VAL_TEAL 75
#define CSS_VAL_WHITE 76
#define CSS_VAL_YELLOW 77
#define CSS_VAL_TRANSPARENT 78
#define CSS_VAL__KHTML_LINK 79
#define CSS_VAL__KHTML_ACTIVELINK 80
#define CSS_VAL_ACTIVEBORDER 81
#define CSS_VAL_ACTIVECAPTION 82
#define CSS_VAL_APPWORKSPACE 83
#define CSS_VAL_BACKGROUND 84
#define CSS_VAL_BUTTONFACE 85
#define CSS_VAL_BUTTONHIGHLIGHT 86
#define CSS_VAL_BUTTONSHADOW 87
#define CSS_VAL_BUTTONTEXT 88
#define CSS_VAL_CAPTIONTEXT 89
#define CSS_VAL_GRAYTEXT 90
#define CSS_VAL_HIGHLIGHT 91
#define CSS_VAL_HIGHLIGHTTEXT 92
#define CSS_VAL_INACTIVEBORDER 93
#define CSS_VAL_INACTIVECAPTION 94
#define CSS_VAL_INACTIVECAPTIONTEXT 95
#define CSS_VAL_INFOBACKGROUND 96
#define CSS_VAL_INFOTEXT 97
#define CSS_VAL_MENUTEXT 98
#define CSS_VAL_SCROLLBAR 99
#define CSS_VAL_THREEDDARKSHADOW 100
#define CSS_VAL_THREEDFACE 101
#define CSS_VAL_THREEDHIGHLIGHT 102
#define CSS_VAL_THREEDLIGHTSHADOW 103
#define CSS_VAL_THREEDSHADOW 104
#define CSS_VAL_WINDOW 105
#define CSS_VAL_WINDOWFRAME 106
#define CSS_VAL_WINDOWTEXT 107
#define CSS_VAL_GREY 108
#define CSS_VAL__KHTML_TEXT 109
#define CSS_VAL_REPEAT 110
#define CSS_VAL_REPEAT_X 111
#define CSS_VAL_REPEAT_Y 112
#define CSS_VAL_NO_REPEAT 113
#define CSS_VAL_BASELINE 114
#define CSS_VAL_MIDDLE 115
#define CSS_VAL_SUB 116
#define CSS_VAL_SUPER 117
#define CSS_VAL_TEXT_TOP 118
#define CSS_VAL_TEXT_BOTTOM 119
#define CSS_VAL_TOP 120
#define CSS_VAL_BOTTOM 121
#define CSS_VAL__KHTML_BASELINE_MIDDLE 122
#define CSS_VAL__KHTML_AUTO 123
#define CSS_VAL_LEFT 124
#define CSS_VAL_RIGHT 125
#define CSS_VAL_CENTER 126
#define CSS_VAL_JUSTIFY 127
#define CSS_VAL__KHTML_LEFT 128
#define CSS_VAL__KHTML_RIGHT 129
#define CSS_VAL__KHTML_CENTER 130
#define CSS_VAL_OUTSIDE 131
#define CSS_VAL_INSIDE 132
#define CSS_VAL_DISC 133
#define CSS_VAL_CIRCLE 134
#define CSS_VAL_SQUARE 135
#define CSS_VAL_DECIMAL 136
#define CSS_VAL_DECIMAL_LEADING_ZERO 137
#define CSS_VAL_LOWER_ROMAN 138
#define CSS_VAL_UPPER_ROMAN 139
#define CSS_VAL_LOWER_GREEK 140
#define CSS_VAL_LOWER_ALPHA 141
#define CSS_VAL_LOWER_LATIN 142
#define CSS_VAL_UPPER_ALPHA 143
#define CSS_VAL_UPPER_LATIN 144
#define CSS_VAL_HEBREW 145
#define CSS_VAL_ARMENIAN 146
#define CSS_VAL_GEORGIAN 147
#define CSS_VAL_CJK_IDEOGRAPHIC 148
#define CSS_VAL_HIRAGANA 149
#define CSS_VAL_KATAKANA 150
#define CSS_VAL_HIRAGANA_IROHA 151
#define CSS_VAL_KATAKANA_IROHA 152
#define CSS_VAL_INLINE 153
#define CSS_VAL_BLOCK 154
#define CSS_VAL_LIST_ITEM 155
#define CSS_VAL_RUN_IN 156
#define CSS_VAL_COMPACT 157
#define CSS_VAL_INLINE_BLOCK 158
#define CSS_VAL_TABLE 159
#define CSS_VAL_INLINE_TABLE 160
#define CSS_VAL_TABLE_ROW_GROUP 161
#define CSS_VAL_TABLE_HEADER_GROUP 162
#define CSS_VAL_TABLE_FOOTER_GROUP 163
#define CSS_VAL_TABLE_ROW 164
#define CSS_VAL_TABLE_COLUMN_GROUP 165
#define CSS_VAL_TABLE_COLUMN 166
#define CSS_VAL_TABLE_CELL 167
#define CSS_VAL_TABLE_CAPTION 168
#define CSS_VAL__KHTML_BOX 169
#define CSS_VAL__KHTML_INLINE_BOX 170
#define CSS_VAL_AUTO 171
#define CSS_VAL_CROSSHAIR 172
#define CSS_VAL_DEFAULT 173
#define CSS_VAL_POINTER 174
#define CSS_VAL_MOVE 175
#define CSS_VAL_E_RESIZE 176
#define CSS_VAL_NE_RESIZE 177
#define CSS_VAL_NW_RESIZE 178
#define CSS_VAL_N_RESIZE 179
#define CSS_VAL_SE_RESIZE 180
#define CSS_VAL_SW_RESIZE 181
#define CSS_VAL_S_RESIZE 182
#define CSS_VAL_W_RESIZE 183
#define CSS_VAL_TEXT 184
#define CSS_VAL_WAIT 185
#define CSS_VAL_HELP 186
#define CSS_VAL_LTR 187
#define CSS_VAL_RTL 188
#define CSS_VAL_CAPITALIZE 189
#define CSS_VAL_UPPERCASE 190
#define CSS_VAL_LOWERCASE 191
#define CSS_VAL_VISIBLE 192
#define CSS_VAL_COLLAPSE 193
#define CSS_VAL_ABOVE 194
#define CSS_VAL_ABSOLUTE 195
#define CSS_VAL_ALWAYS 196
#define CSS_VAL_AVOID 197
#define CSS_VAL_BELOW 198
#define CSS_VAL_BIDI_OVERRIDE 199
#define CSS_VAL_BLINK 200
#define CSS_VAL_BOTH 201
#define CSS_VAL_CLOSE_QUOTE 202
#define CSS_VAL_CROP 203
#define CSS_VAL_CROSS 204
#define CSS_VAL_EMBED 205
#define CSS_VAL_FIXED 206
#define CSS_VAL_HAND 207
#define CSS_VAL_HIDE 208
#define CSS_VAL_HIGHER 209
#define CSS_VAL_INVERT 210
#define CSS_VAL_LANDSCAPE 211
#define CSS_VAL_LEVEL 212
#define CSS_VAL_LINE_THROUGH 213
#define CSS_VAL_LOUD 214
#define CSS_VAL_LOWER 215
#define CSS_VAL_MARQUEE 216
#define CSS_VAL_MIX 217
#define CSS_VAL_NO_CLOSE_QUOTE 218
#define CSS_VAL_NO_OPEN_QUOTE 219
#define CSS_VAL_NOWRAP 220
#define CSS_VAL_OPEN_QUOTE 221
#define CSS_VAL_OVERLAY 222
#define CSS_VAL_OVERLINE 223
#define CSS_VAL_PORTRAIT 224
#define CSS_VAL_PRE 225
#define CSS_VAL_RELATIVE 226
#define CSS_VAL_SCROLL 227
#define CSS_VAL_SEPARATE 228
#define CSS_VAL_SHOW 229
#define CSS_VAL_STATIC 230
#define CSS_VAL_THICK 231
#define CSS_VAL_THIN 232
#define CSS_VAL_UNDERLINE 233
#define CSS_VAL__KHTML_NOWRAP 234
#define CSS_VAL_STRETCH 235
#define CSS_VAL_START 236
#define CSS_VAL_END 237
#define CSS_VAL_REVERSE 238
#define CSS_VAL_HORIZONTAL 239
#define CSS_VAL_VERTICAL 240
#define CSS_VAL_INLINE_AXIS 241
#define CSS_VAL_BLOCK_AXIS 242
#define CSS_VAL_SINGLE 243
#define CSS_VAL_MULTIPLE 244
#define CSS_VAL_FORWARDS 245
#define CSS_VAL_BACKWARDS 246
#define CSS_VAL_AHEAD 247
#define CSS_VAL_UP 248
#define CSS_VAL_DOWN 249
#define CSS_VAL_SLOW 250
#define CSS_VAL_FAST 251
#define CSS_VAL_INFINITE 252
#define CSS_VAL_SLIDE 253
#define CSS_VAL_ALTERNATE 254
#define CSS_VAL_UNFURL 255
#define CSS_VAL_READ_ONLY 256
#define CSS_VAL_READ_WRITE 257
#define CSS_VAL_ELEMENT 258
#define CSS_VAL_INTRINSIC 259
#define CSS_VAL_MIN_INTRINSIC 260
#define CSS_VAL_CLIP 261
#define CSS_VAL_ELLIPSIS 262

#define CSS_VAL_TOTAL 263
#endif

