/* This file is automatically generated from
htmltags.in by maketags, do not edit */
/* Copyright 1999 Lars Knoll */

#ifndef KHTML_TAGS_H
#define KHTML_TAGS_H

#include "dom/dom_string.h"

DOM::DOMString getTagName(unsigned short id);
#define ID_A 1
#define ID_ABBR 2
#define ID_ACRONYM 3
#define ID_ADDRESS 4
#define ID_APPLET 5
#define ID_AREA 6
#define ID_B 7
#define ID_BASE 8
#define ID_BASEFONT 9
#define ID_BDO 10
#define ID_BIG 11
#define ID_BLOCKQUOTE 12
#define ID_BODY 13
#define ID_BR 14
#define ID_BUTTON 15
#define ID_CANVAS 16
#define ID_CAPTION 17
#define ID_CENTER 18
#define ID_CITE 19
#define ID_CODE 20
#define ID_COL 21
#define ID_COLGROUP 22
#define ID_DD 23
#define ID_DEL 24
#define ID_DFN 25
#define ID_DIR 26
#define ID_DIV 27
#define ID_DL 28
#define ID_DT 29
#define ID_EM 30
#define ID_EMBED 31
#define ID_FIELDSET 32
#define ID_FONT 33
#define ID_FORM 34
#define ID_FRAME 35
#define ID_FRAMESET 36
#define ID_H1 37
#define ID_H2 38
#define ID_H3 39
#define ID_H4 40
#define ID_H5 41
#define ID_H6 42
#define ID_HEAD 43
#define ID_HR 44
#define ID_HTML 45
#define ID_I 46
#define ID_IFRAME 47
#define ID_IMG 48
#define ID_INPUT 49
#define ID_INS 50
#define ID_ISINDEX 51
#define ID_KBD 52
#define ID_KEYGEN 53
#define ID_LABEL 54
#define ID_LAYER 55
#define ID_LEGEND 56
#define ID_LI 57
#define ID_LINK 58
#define ID_MAP 59
#define ID_MARQUEE 60
#define ID_MENU 61
#define ID_META 62
#define ID_NOBR 63
#define ID_NOEMBED 64
#define ID_NOFRAMES 65
#define ID_NOSCRIPT 66
#define ID_NOLAYER 67
#define ID_OBJECT 68
#define ID_OL 69
#define ID_OPTGROUP 70
#define ID_OPTION 71
#define ID_P 72
#define ID_PARAM 73
#define ID_PLAINTEXT 74
#define ID_PRE 75
#define ID_Q 76
#define ID_S 77
#define ID_SAMP 78
#define ID_SCRIPT 79
#define ID_SELECT 80
#define ID_SMALL 81
#define ID_SPAN 82
#define ID_STRIKE 83
#define ID_STRONG 84
#define ID_STYLE 85
#define ID_SUB 86
#define ID_SUP 87
#define ID_TABLE 88
#define ID_TBODY 89
#define ID_TD 90
#define ID_TEXTAREA 91
#define ID_TFOOT 92
#define ID_TH 93
#define ID_THEAD 94
#define ID_TITLE 95
#define ID_TR 96
#define ID_TT 97
#define ID_U 98
#define ID_UL 99
#define ID_VAR 100
#define ID_WBR 101
#define ID_XMP 102
#define ID_TEXT 103
#define ID_COMMENT 104
#define ID_CLOSE_TAG 104
#define ID_LAST_TAG 104

#endif
