/* This file is automatically generated from cssproperties.in by makeprop, do not edit */
/* Copyright 1998 W. Bastian */

#ifndef CSSPROPERTIES_H
#define CSSPROPERTIES_H


#define CSS_PROP_INVALID 0
#define CSS_PROP_MIN 1
#define CSS_PROP_BACKGROUND_COLOR 1
#define CSS_PROP_BACKGROUND_IMAGE 2
#define CSS_PROP_BACKGROUND_REPEAT 3
#define CSS_PROP_BACKGROUND_ATTACHMENT 4
#define CSS_PROP_BACKGROUND_POSITION 5
#define CSS_PROP_BACKGROUND_POSITION_X 6
#define CSS_PROP_BACKGROUND_POSITION_Y 7
#define CSS_PROP__KHTML_BINDING 8
#define CSS_PROP_BORDER_COLLAPSE 9
#define CSS_PROP_BORDER_SPACING 10
#define CSS_PROP__KHTML_BORDER_HORIZONTAL_SPACING 11
#define CSS_PROP__KHTML_BORDER_VERTICAL_SPACING 12
#define CSS_PROP_BORDER_TOP_COLOR 13
#define CSS_PROP_BORDER_RIGHT_COLOR 14
#define CSS_PROP_BORDER_BOTTOM_COLOR 15
#define CSS_PROP_BORDER_LEFT_COLOR 16
#define CSS_PROP_BORDER_TOP_STYLE 17
#define CSS_PROP_BORDER_RIGHT_STYLE 18
#define CSS_PROP_BORDER_BOTTOM_STYLE 19
#define CSS_PROP_BORDER_LEFT_STYLE 20
#define CSS_PROP_BORDER_TOP_WIDTH 21
#define CSS_PROP_BORDER_RIGHT_WIDTH 22
#define CSS_PROP_BORDER_BOTTOM_WIDTH 23
#define CSS_PROP_BORDER_LEFT_WIDTH 24
#define CSS_PROP_BOTTOM 25
#define CSS_PROP__KHTML_BOX_ALIGN 26
#define CSS_PROP__KHTML_BOX_DIRECTION 27
#define CSS_PROP__KHTML_BOX_FLEX 28
#define CSS_PROP__KHTML_BOX_FLEX_GROUP 29
#define CSS_PROP__KHTML_BOX_LINES 30
#define CSS_PROP__KHTML_BOX_ORDINAL_GROUP 31
#define CSS_PROP__KHTML_BOX_ORIENT 32
#define CSS_PROP__KHTML_BOX_PACK 33
#define CSS_PROP_CAPTION_SIDE 34
#define CSS_PROP_CLEAR 35
#define CSS_PROP_CLIP 36
#define CSS_PROP_COLOR 37
#define CSS_PROP_CONTENT 38
#define CSS_PROP_COUNTER_INCREMENT 39
#define CSS_PROP_COUNTER_RESET 40
#define CSS_PROP_CURSOR 41
#define CSS_PROP_DIRECTION 42
#define CSS_PROP_DISPLAY 43
#define CSS_PROP_EMPTY_CELLS 44
#define CSS_PROP_FLOAT 45
#define CSS_PROP_FONT_FAMILY 46
#define CSS_PROP_FONT_SIZE 47
#define CSS_PROP_FONT_SIZE_ADJUST 48
#define CSS_PROP_FONT_STRETCH 49
#define CSS_PROP_FONT_STYLE 50
#define CSS_PROP_FONT_VARIANT 51
#define CSS_PROP_FONT_WEIGHT 52
#define CSS_PROP_HEIGHT 53
#define CSS_PROP_LEFT 54
#define CSS_PROP_LETTER_SPACING 55
#define CSS_PROP__APPLE_LINE_CLAMP 56
#define CSS_PROP_LINE_HEIGHT 57
#define CSS_PROP_LIST_STYLE_IMAGE 58
#define CSS_PROP_LIST_STYLE_POSITION 59
#define CSS_PROP_LIST_STYLE_TYPE 60
#define CSS_PROP_MARGIN_TOP 61
#define CSS_PROP_MARGIN_RIGHT 62
#define CSS_PROP_MARGIN_BOTTOM 63
#define CSS_PROP_MARGIN_LEFT 64
#define CSS_PROP__KHTML_MARQUEE 65
#define CSS_PROP__KHTML_MARQUEE_DIRECTION 66
#define CSS_PROP__KHTML_MARQUEE_INCREMENT 67
#define CSS_PROP__KHTML_MARQUEE_REPETITION 68
#define CSS_PROP__KHTML_MARQUEE_SPEED 69
#define CSS_PROP__KHTML_MARQUEE_STYLE 70
#define CSS_PROP_MAX_HEIGHT 71
#define CSS_PROP_MAX_WIDTH 72
#define CSS_PROP_MIN_HEIGHT 73
#define CSS_PROP_MIN_WIDTH 74
#define CSS_PROP_OPACITY 75
#define CSS_PROP_ORPHANS 76
#define CSS_PROP_OUTLINE_COLOR 77
#define CSS_PROP_OUTLINE_OFFSET 78
#define CSS_PROP_OUTLINE_STYLE 79
#define CSS_PROP_OUTLINE_WIDTH 80
#define CSS_PROP_OVERFLOW 81
#define CSS_PROP_PADDING_TOP 82
#define CSS_PROP_PADDING_RIGHT 83
#define CSS_PROP_PADDING_BOTTOM 84
#define CSS_PROP_PADDING_LEFT 85
#define CSS_PROP_PAGE 86
#define CSS_PROP_PAGE_BREAK_AFTER 87
#define CSS_PROP_PAGE_BREAK_BEFORE 88
#define CSS_PROP_PAGE_BREAK_INSIDE 89
#define CSS_PROP_POSITION 90
#define CSS_PROP_QUOTES 91
#define CSS_PROP_RIGHT 92
#define CSS_PROP_SIZE 93
#define CSS_PROP_TABLE_LAYOUT 94
#define CSS_PROP_TEXT_ALIGN 95
#define CSS_PROP_TEXT_DECORATION 96
#define CSS_PROP_TEXT_DECORATION_COLOR 97
#define CSS_PROP_TEXT_INDENT 98
#define CSS_PROP_TEXT_OVERFLOW 99
#define CSS_PROP_TEXT_SHADOW 100
#define CSS_PROP_TEXT_TRANSFORM 101
#define CSS_PROP__APPLE_TEXT_SIZE_ADJUST 102
#define CSS_PROP_TOP 103
#define CSS_PROP_UNICODE_BIDI 104
#define CSS_PROP__KHTML_USER_DRAG 105
#define CSS_PROP__KHTML_USER_MODIFY 106
#define CSS_PROP__KHTML_USER_SELECT 107
#define CSS_PROP_VERTICAL_ALIGN 108
#define CSS_PROP_VISIBILITY 109
#define CSS_PROP_WHITE_SPACE 110
#define CSS_PROP_WIDOWS 111
#define CSS_PROP_WIDTH 112
#define CSS_PROP_WORD_SPACING 113
#define CSS_PROP_Z_INDEX 114
#define CSS_PROP_BACKGROUND 115
#define CSS_PROP_BORDER 116
#define CSS_PROP_BORDER_COLOR 117
#define CSS_PROP_BORDER_STYLE 118
#define CSS_PROP_BORDER_TOP 119
#define CSS_PROP_BORDER_RIGHT 120
#define CSS_PROP_BORDER_BOTTOM 121
#define CSS_PROP_BORDER_LEFT 122
#define CSS_PROP_BORDER_WIDTH 123
#define CSS_PROP_FONT 124
#define CSS_PROP_LIST_STYLE 125
#define CSS_PROP_MARGIN 126
#define CSS_PROP_OUTLINE 127
#define CSS_PROP_PADDING 128
#define CSS_PROP_SCROLLBAR_FACE_COLOR 129
#define CSS_PROP_SCROLLBAR_SHADOW_COLOR 130
#define CSS_PROP_SCROLLBAR_HIGHLIGHT_COLOR 131
#define CSS_PROP_SCROLLBAR_3DLIGHT_COLOR 132
#define CSS_PROP_SCROLLBAR_DARKSHADOW_COLOR 133
#define CSS_PROP_SCROLLBAR_TRACK_COLOR 134
#define CSS_PROP_SCROLLBAR_ARROW_COLOR 135
#define CSS_PROP__KHTML_FLOW_MODE 136

#define CSS_PROP_MAX CSS_PROP_Z_INDEX
#define CSS_PROP_TOTAL 137
#endif

